--- src/enblend.cc.orig	2017-02-13 07:55:01 UTC
+++ src/enblend.cc
@@ -119,6 +119,7 @@ std::optional<std::string> OutputMaskFil
 int Verbose = 0;                //< default-verbosity-level 0
 int ExactLevels = 0;            // 0 means: automatically calculate maximum
 bool OneAtATime = true;
+bool CopyMetadata = true;
 boundary_t WrapAround = OpenBoundaries;
 bool GimpAssociatedAlphaHack = false;
 blend_colorspace_t BlendColorspace = UndeterminedColorspace;
@@ -240,10 +241,11 @@ void dump_global_variables(const char* f
         "+ " << file << ":" << line << ": state of global variables\n" <<
         "+ Verbose = " << Verbose << ", option \"--verbose\"\n" <<
         "+ OutputFileName = <" << OutputFileName << ">\n" <<
-        "+ OutputMaskFileName = <" << OutputMaskFileName.value_or("<not defined>") << ">\n" <<
+        "+ OutputMaskFileName = <" << OutputMaskFileName.get_value_or("<not defined>") << ">\n" <<
         "+ ExactLevels = " << ExactLevels << "\n" <<
         "+ UseGPU = " << UseGPU << "\n" <<
         "+ OneAtATime = " << enblend::stringOfBool(OneAtATime) << ", option \"-a\"\n" <<
+        "+ CopyMetadata = " << enblend::stringOfBool(CopyMetadata) << ", option \"--[no-]metadata\"\n" <<
         "+ WrapAround = " << enblend::stringOfWraparound(WrapAround) << ", option \"--wrap\"\n" <<
         "+ GimpAssociatedAlphaHack = " << enblend::stringOfBool(GimpAssociatedAlphaHack) <<
         ", option \"-g\"\n" <<
@@ -491,7 +493,8 @@ void sigint_handler(int sig)
 
 
 enum AllPossibleOptions {
-    VersionOption, PreAssembleOption /* -a */, NoPreAssembleOption, HelpOption, LevelsOption,
+    VersionOption, PreAssembleOption /* -a */, NoPreAssembleOption,
+    MetadataOption, NoMetadataOption, HelpOption, LevelsOption,
     OutputOption, OutputMaskOption, VerboseOption, WrapAroundOption /* -w */,
     CheckpointOption /* -x */, CompressionOption, LZWCompressionOption,
     BlendColorspaceOption, FallbackProfileOption,
@@ -519,6 +522,12 @@ bool contains(const OptionSetType& optio
 }
 
 
+static void lcmsErrorHandler(cmsContext context, cmsUInt32Number errorCode, char const *errorText)
+{
+    std::cerr << "lcms: error (" << errorCode << "): " << errorText << std::endl;
+}
+
+
 /** Warn if options given at the command line have no effect. */
 void warn_of_ineffective_options(const OptionSetType& optionSet)
 {
@@ -768,6 +777,8 @@ process_options(int argc, char** argv)
         PreferGpuId,
         PreAssembleId,
         NoPreAssembleId,
+        MetadataId,
+        NoMetadataId,
         CoarseMaskId,
         FineMaskId,
         OptimizeMaskId,
@@ -811,6 +822,8 @@ process_options(int argc, char** argv)
         {"preassemble", no_argument, 0, PreAssembleId}, // dash-less form: not documented, not deprecated
         {"no-pre-assemble", no_argument, 0, NoPreAssembleId},
         {"no-preassemble", no_argument, 0, NoPreAssembleId}, // dash-less form: not documented, not deprecated
+        {"metadata", no_argument, 0, MetadataId},
+        {"no-metadata", no_argument, 0, NoMetadataId},
         {"coarse-mask", optional_argument, 0, CoarseMaskId},
         {"fine-mask", no_argument, 0, FineMaskId},
         {"optimize", no_argument, 0, OptimizeMaskId},
@@ -1400,6 +1413,16 @@ process_options(int argc, char** argv)
             optionSet.insert(NoPreAssembleOption);
             break;
 
+        case MetadataId:
+            CopyMetadata = true;
+            optionSet.insert(MetadataOption);
+            break;
+
+        case NoMetadataId:
+            CopyMetadata = false;
+            optionSet.insert(NoMetadataOption);
+            break;
+
         case BlendColorspaceId:
             if (optarg != nullptr && *optarg != 0) {
                 std::string name(optarg);
@@ -1412,6 +1435,8 @@ process_options(int argc, char** argv)
                     BlendColorspace = CIELUV;
                 } else if (name == "CIECAM" || name == "CIECAM02" || name == "JCH") {
                     BlendColorspace = CIECAM;
+                } else if (name == "XYZ") {
+                    BlendColorspace = XYZ;
                 } else {
                     std::cerr << command <<
                         ": unrecognized argument \"" << optarg << "\" of option \"--blend-colorspace\"" <<
@@ -1762,6 +1787,8 @@ int main(int argc, char** argv)
     //< layer-selector all-layers
     LayerSelection.set_selector(selector::find_by_id(selector::id_t::AllLayersId)->get());
 
+    cmsSetLogErrorHandler(lcmsErrorHandler);
+
     if (!getopt_long_works_ok())
     {
         std::cerr << command << ": cannot reliably parse command line; giving up\n";
@@ -2182,13 +2209,15 @@ int main(int argc, char** argv)
         // Set the output image ICC profile
         outputImageInfo.setICCProfile(iccProfile);
 
+        bool const isfp(enblend::isFloatingPoint(pixelType));
+
         if (BlendColorspace == UndeterminedColorspace &&
-            !(iccProfile.empty() || enblend::isFloatingPoint(pixelType))) {
+            !(iccProfile.empty() || isfp)) {
             BlendColorspace = CIELUV;
         }
 
-        if (BlendColorspace == CIECAM || BlendColorspace == CIELAB || BlendColorspace == CIELUV) {
-            if (enblend::isFloatingPoint(pixelType)) {
+        if (BlendColorspace == CIECAM || BlendColorspace == CIELAB || BlendColorspace == CIELUV || BlendColorspace == XYZ) {
+            if (isfp) {
                 std::cerr << command <<
                     ": warning: blend color space for floating-point images is not \"identity\"" << std::endl;
             }
@@ -2196,8 +2225,28 @@ int main(int argc, char** argv)
             if (InputProfile == nullptr) {
                 std::cerr << command << ": warning: input images do not have ICC profiles;\n";
                 if (FallbackProfile == nullptr) {
-                    std::cerr << command << ": warning: assuming sRGB profile" << std::endl;
-                    InputProfile = cmsCreate_sRGBProfile();
+                    std::cerr << command << ": warning: assuming " << (isfp ? "RGB" : "sRGB") << " profile" << std::endl;
+                    if (isfp) {
+                        static const cmsCIExyY wpSpec50Illuminant = {
+                            0.345702915,  0.358538597,  1.0
+                        };
+                        static const cmsCIExyYTRIPLE primariesIdentity = {
+                            {1.000000000,  0.000000000,  1.0},
+                            {0.000000000,  1.000000000,  1.0},
+                            {0.000000000,  0.000000000,  1.0}
+                        };
+                        static uint16_t const linear_curve[] = {65535};
+                        static size_t const linear_curve_n =
+                            sizeof(linear_curve)/sizeof(linear_curve[0]);
+                        cmsToneCurve *transferFunction;
+                        cmsToneCurve *transferFunctions[3];
+                        transferFunction = cmsBuildTabulatedToneCurve16(NULL, linear_curve_n, linear_curve);
+                        transferFunctions[0] = transferFunctions[1] = transferFunctions[2] = transferFunction;
+                        InputProfile = cmsCreateRGBProfile(&wpSpec50Illuminant, &primariesIdentity, transferFunctions); 
+                        cmsFreeToneCurve(transferFunction);
+                    } else {
+                        InputProfile = cmsCreate_sRGBProfile();
+                    }
                 } else {
                     std::cerr << command << ": warning: using fallback profile \""
                               << enblend::profileDescription(FallbackProfile) << "\"" << std::endl;
