--- src/fixmath.h.orig	2017-04-05 14:47:21 UTC
+++ src/fixmath.h
@@ -44,7 +44,8 @@
 
 // ANTICIPATED CHANGE: Remove this define after Lab/Luv optimization
 // has gotten enough testing.
-#define LOG_COLORSPACE_OPTIMIZATION
+//#define LOG_COLORSPACE_OPTIMIZATION
+//#define LOG_COLORSPACE_CONVERSION
 
 
 #define XYZ_SCALE 100.0
@@ -389,7 +390,8 @@ public:
 
     PyramidVectorType operator()(const SrcVectorType& v) const
     {
-        return PyramidVectorType(converter(v.red()), converter(v.green()), converter(v.blue()));
+        //return PyramidVectorType(converter(v.red()), converter(v.green()), converter(v.blue()));
+        return PyramidVectorType(converter(v.green()), converter(v.blue()), converter(v.red()));
     }
 
 protected:
@@ -414,7 +416,8 @@ public:
 
     DestVectorType operator()(const PyramidVectorType& v) const
     {
-        return DestVectorType(converter(v.red()), converter(v.green()), converter(v.blue()));
+        //return DestVectorType(converter(v.red()), converter(v.green()), converter(v.blue()));
+        return DestVectorType(converter(v.blue()), converter(v.red()), converter(v.green()));
     }
 
 protected:
@@ -467,7 +470,7 @@ class PyramidScale
 public:
     PyramidScale() :
         color_limit(parameter::as_double("lab-color-limit", 200.0)),
-        pyramid_scale(double(1U << (PyramidIntegerBits - PYRAMID_HEADROOM_BITS)))
+        pyramid_scale(double(1LU << (PyramidIntegerBits - PYRAMID_HEADROOM_BITS)))
     {
         static_assert(PyramidIntegerBits >= PYRAMID_HEADROOM_BITS,
                       "not enough PyramidIntegerBits to calculate `pyramid_scale' from shift");
@@ -1344,7 +1347,7 @@ public:
     };
 
     PyramidScaleJCh() :
-        pyramid_scale(double(1U << (PyramidIntegerBits - 1 - 7)))
+        pyramid_scale(double(1LU << (PyramidIntegerBits - 1 - 7)))
     {
         static_assert(PyramidIntegerBits >= 8,
                       "not enough PyramidIntegerBits to get `pyramid_scale' from shift");
@@ -2008,6 +2011,253 @@ protected:
 };
 
 
+//
+//  Fixed point converter that uses ICC profile transformation and XYZ color space
+//
+
+template <typename SrcVectorType, typename PyramidVectorType, int PyramidIntegerBits, int PyramidFractionBits>
+class ConvertVectorToXYZPyramidFunctor :
+    private PyramidScale<PyramidIntegerBits, PyramidFractionBits>
+{
+    //typedef PyramidScale<PyramidIntegerBits, PyramidFractionBits> Scale;
+    typedef typename SrcVectorType::value_type SrcComponentType;
+    typedef vigra::NumericTraits<SrcComponentType> SrcTraits;
+    typedef typename PyramidVectorType::value_type PyramidComponentType;
+    typedef ConvertScalarToPyramidFunctor<SrcComponentType, PyramidComponentType,
+                                          PyramidIntegerBits, PyramidFractionBits> ConvertFunctorType;
+    //typedef vigra::XYZ2LuvFunctor<double> XYZ2LuvFunctor;
+
+public:
+    ConvertVectorToXYZPyramidFunctor() :
+        converter(),
+        //rgb_source_scale(1.0 / SrcTraits::toRealPromote(SrcTraits::max()))
+        rgb_source_scale(1.0)
+#ifdef LOG_COLORSPACE_CONVERSION
+        , range("X", "Y", "Z")
+#endif // LOG_COLORSPACE_CONVERSION
+    {
+#ifdef LOG_COLORSPACE_CONVERSION
+#ifdef OPENMP
+#pragma omp critical
+#endif
+        std::cout <<
+            "+ ConvertVectorToXYZPyramidFunctor::ConvertVectorToXYZPyramidFunctor: source range [" <<
+            static_cast<double>(SrcTraits::min()) << ", " <<
+            static_cast<double>(SrcTraits::max()) << "]\n" <<
+            "+ ConvertVectorToXYZPyramidFunctor::ConvertVectorToXYZPyramidFunctor: pyramid range [" <<
+            static_cast<double>(vigra::NumericTraits<PyramidComponentType>::min()) << ", " <<
+            static_cast<double>(vigra::NumericTraits<PyramidComponentType>::max()) << "]" << std::endl;
+#endif // LOG_COLORSPACE_CONVERSION
+    }
+
+#ifdef LOG_COLORSPACE_CONVERSION
+    ~ConvertVectorToXYZPyramidFunctor()
+    {
+#ifdef OPENMP
+#pragma omp critical
+#endif
+        std::cout <<
+            range.as_string("+ ConvertVectorToXYZPyramidFunctor::~ConvertVectorToXYZPyramidFunctor: ") <<
+            std::endl;
+    }
+#endif // LOG_COLORSPACE_CONVERSION
+
+    PyramidVectorType operator()(const SrcVectorType& v) const
+    {
+        static double const rgb2xyzD65_srgb[3][3] = {
+            {  0.4124564,  0.3575761,  0.1804375 },
+            {  0.2126729,  0.7151522,  0.0721750 },
+            {  0.0193339,  0.1191920,  0.9503041 }
+        };
+        static double const rgb2xyzD50_srgb[3][3] = {
+            {  0.4360747,  0.3850649,  0.1430804 },
+            {  0.2225045,  0.7168786,  0.0606169 },
+            {  0.0139322,  0.0971045,  0.7141733 }
+        };
+        static double const rgb2xyzD50_wide_gamut[3][3] = {
+            {  0.7161046,  0.1009296,  0.1471858 },
+            {  0.2581874,  0.7249378,  0.0168748 },
+            {  0.0000000,  0.0517813,  0.7734287 }
+        };
+        static double const rgb2xyzD50_identity[3][3] = {
+            {  0.9642200,  0.0000000,  0.0000000 },
+            {  0.0000000,  1.0000000,  0.0000000 },
+            {  0.0000000,  0.0000000,  0.8252100 }
+        };
+#define rgb2xyz rgb2xyzD65_srgb
+        const double rgb[] = {
+            rgb_source_scale * SrcTraits::toRealPromote(v.red()),
+            rgb_source_scale * SrcTraits::toRealPromote(v.green()),
+            rgb_source_scale * SrcTraits::toRealPromote(v.blue())
+        };
+        const double xyz[] = {
+            rgb2xyz[0][0]*rgb[0] + rgb2xyz[0][1]*rgb[0] + rgb2xyz[0][2]*rgb[0], 
+            rgb2xyz[1][0]*rgb[1] + rgb2xyz[1][1]*rgb[1] + rgb2xyz[1][2]*rgb[1], 
+            rgb2xyz[2][0]*rgb[2] + rgb2xyz[2][1]*rgb[2] + rgb2xyz[2][2]*rgb[2]
+        };
+        /*XYZ2LuvFunctor::argument_type xyz;
+
+        cmsDoTransform(InputToXYZTransform, rgb, &xyz[0], 1U);
+#ifdef LOG_COLORSPACE_CONVERSION
+        range.update(xyz[0], xyz[1], xyz[2]);
+#endif // LOG_COLORSPACE_CONVERSION
+
+        return PyramidVectorType(converter(Scale::scale_color_difference_for_pyramid(xyz[0])),
+                                 converter(Scale::scale_lightness_for_pyramid(xyz[1])),
+                                 converter(Scale::scale_color_difference_for_pyramid(xyz[2])));*/
+
+        return PyramidVectorType(converter(xyz[0]), converter(xyz[1]), converter(xyz[2]));
+    }
+
+protected:
+    ConvertFunctorType converter;
+    const double rgb_source_scale;
+#ifdef LOG_COLORSPACE_CONVERSION
+    mutable TriplePeakHold range;
+#endif // LOG_COLORSPACE_CONVERSION
+};
+
+
+//
+// Fixed point converter that uses ICC profile transformation and XYZ color space
+//
+
+template <typename DestVectorType, typename PyramidVectorType, int PyramidIntegerBits, int PyramidFractionBits>
+class ConvertXYZPyramidToVectorFunctor :
+    private PyramidScale<PyramidIntegerBits, PyramidFractionBits>/*,
+    private OptimizableLuminanceSpace*/
+{
+    //typedef PyramidScale<PyramidIntegerBits, PyramidFractionBits> Scale;
+    typedef typename DestVectorType::value_type DestComponentType;
+    typedef vigra::NumericTraits<DestComponentType> DestTraits;
+    typedef typename PyramidVectorType::value_type PyramidComponentType;
+    typedef ConvertPyramidToScalarFunctor<DestComponentType, PyramidComponentType,
+                                          PyramidIntegerBits, PyramidFractionBits> ConvertFunctorType;
+    //typedef vigra::XYZ2LabFunctor<double> XYZ2LabFunctor;
+
+public:
+    ConvertXYZPyramidToVectorFunctor() :
+        converter(),
+        //rgb_dest_scale(DestTraits::toRealPromote(DestTraits::max()))
+        rgb_dest_scale(1.0)
+#ifdef LOG_COLORSPACE_CONVERSION
+        , range("X", "Y", "Z")
+#endif // LOG_COLORSPACE_CONVERSION
+    {
+#ifdef LOG_COLORSPACE_CONVERSION
+#ifdef OPENMP
+#pragma omp critical
+#endif
+        std::cout <<
+            "+ ConvertXYZPyramidToVectorFunctor::ConvertXYZPyramidToVectorFunctor: dest range [" <<
+            static_cast<double>(DestTraits::min()) << ", " <<
+            static_cast<double>(DestTraits::max()) << "]\n" <<
+            "+ ConvertXYZPyramidToVectorFunctor::ConvertXYZPyramidToVectorFunctor: pyramid range [" <<
+            static_cast<double>(vigra::NumericTraits<PyramidComponentType>::min()) << ", " <<
+            static_cast<double>(vigra::NumericTraits<PyramidComponentType>::max()) << "]" << std::endl;
+#endif // LOG_COLORSPACE_CONVERSION
+    }
+
+#ifdef LOG_COLORSPACE_CONVERSION
+    ~ConvertXYZPyramidToVectorFunctor()
+    {
+#ifdef OPENMP
+#pragma omp critical
+#endif
+        std::cout <<
+            range.as_string("+ ConvertXYZPyramidToVectorFunctor::~ConvertXYZPyramidToVectorFunctor: ") <<
+            std::endl;
+    }
+#endif // LOG_COLORSPACE_CONVERSION
+
+    DestVectorType operator()(const PyramidVectorType& v) const
+    {
+        static double const xyz2rgbD65_srgb[3][3] = {
+            {  3.2404542, -1.5371385, -0.4985314 },
+            { -0.9692660,  1.8760108,  0.0415560 },
+            {  0.0556434, -0.2040259,  1.0572252 }
+        };
+        static double const xyz2rgb[3][3] = {
+            {  3.1338561, -1.6168667, -0.4906146 },
+            { -0.9787684,  1.9161415,  0.0334540 },
+            {  0.0719453, -0.2289914,  1.4052427 }
+        };
+        static double const xyz2rgbD50_wide_gamut[3][3] = {
+            {  1.4628067, -0.1840623, -0.2743606 },
+            { -0.5217933,  1.4472381,  0.0677227 },
+            {  0.0349342, -0.0968930,  1.2884099 }
+        };
+        static double const xyz2rgbD50_identity[3][3] = {
+            {  1.0371077,  0.0000000,  0.0000000 },
+            {  0.0000000,  1.0000000,  0.0000000 },
+            {  0.0000000,  0.0000000,  1.2118128 }
+        };
+#define xyz2rgb xyz2rgbD65_srgb
+        /*const XYZ2LabFunctor::argument_type xyz*/
+        /*double xyz[] {
+            Scale::scale_color_difference_of_pyramid(converter(v.red())),
+            Scale::scale_lightness_of_pyramid(converter(v.green())),
+            Scale::scale_color_difference_of_pyramid(converter(v.blue()))
+        };*/
+        double xyz[] {
+            static_cast<double>(converter(v.red())),
+            static_cast<double>(converter(v.green())),
+            static_cast<double>(converter(v.blue()))
+        };
+#ifdef LOG_COLORSPACE_CONVERSION
+        range.update(xyz[0], xyz[1], xyz[2]);
+#endif // LOG_COLORSPACE_CONVERSION
+
+        //assert(!std::isnan(xyz[1]) && xyz[1] >= 0.0);
+        if (std::isnan(xyz[1]) || xyz[1] < 0.0)
+        {
+//#ifdef LOG_COLORSPACE_CONVERSION
+#ifdef OPENMP
+#pragma omp critical
+#endif
+            std::cerr << "X: " << xyz[0] << " ==> 0.0; "
+                      << "Y: " << xyz[1] << " ==> 0.0; "
+                      << "Z: " << xyz[2] << " ==> 0.0"
+                      << std::endl;
+//#endif // LOG_COLORSPACE_CONVERSION
+            xyz[0] = xyz[1] = xyz[2] = 0.0;
+        }
+        const double rgb[] = {
+            xyz2rgb[0][0]*xyz[0] + xyz2rgb[0][1]*xyz[0] + xyz2rgb[0][2]*xyz[0], 
+            xyz2rgb[1][0]*xyz[1] + xyz2rgb[1][1]*xyz[1] + xyz2rgb[1][2]*xyz[1], 
+            xyz2rgb[2][0]*xyz[2] + xyz2rgb[2][1]*xyz[2] + xyz2rgb[2][2]*xyz[2]
+        };
+        /*double rgb[3];
+
+        cmsDoTransform(XYZToInputTransform, xyz, rgb, 1U);*/
+
+        /*if (EXPECT_RESULT(is_below_threshold(rgb), false))
+        {
+            // Optimization Possibility: If XYZ2LabFunctor is instantiated with `double':
+            //     union {
+            //         XYZ2LabFunctor::result_type lab_array;
+            //         cmsCIELab lab_struct;
+            //     };
+            const XYZ2LabFunctor::result_type lab_vector {xyz2lab(xyz)};
+            const cmsCIELab lab {lab_vector[0], lab_vector[1], lab_vector[2]};
+            polish_rgb(&lab, rgb);
+        }*/
+
+        return DestVectorType(DestTraits::fromRealPromote(rgb_dest_scale * rgb[0]),
+                              DestTraits::fromRealPromote(rgb_dest_scale * rgb[1]),
+                              DestTraits::fromRealPromote(rgb_dest_scale * rgb[2]));
+    }
+
+protected:
+    //XYZ2LabFunctor xyz2lab;
+    ConvertFunctorType converter;
+    const double rgb_dest_scale;
+#ifdef LOG_COLORSPACE_CONVERSION
+    mutable TriplePeakHold range;
+#endif // LOG_COLORSPACE_CONVERSION
+};
+
+
 ////////////////////////////////////////////////////////////////////////////////////////////////
 //
 // Copy TO pyramid
@@ -2056,6 +2306,8 @@ copyToPyramidImage(typename SrcImageType
                                              PyramidIntegerBits, PyramidFractionBits> ConverterLuv;
     typedef ConvertVectorToJCHPyramidFunctor<SrcVectorType, PyramidVectorType,
                                              PyramidIntegerBits, PyramidFractionBits> ConverterJCH;
+    typedef ConvertVectorToXYZPyramidFunctor<SrcVectorType, PyramidVectorType,
+                                             PyramidIntegerBits, PyramidFractionBits> ConverterXYZ;
 
     switch (BlendColorspace)
     {
@@ -2111,6 +2363,21 @@ copyToPyramidImage(typename SrcImageType
                                    ConverterJCH());
         break;
 
+    case XYZ:
+        if (Verbose >= VERBOSE_COLOR_CONVERSION_MESSAGES)
+        {
+            std::cerr << command << ": info: XYZ color conversion";
+            if (!enblend::profileName(InputProfile).empty())
+            {
+                std::cerr << " from/to \"" << enblend::profileName(InputProfile) << "\" profile";
+            }
+            std::cerr << "\n";
+        }
+        vigra::omp::transformImage(src_upperleft, src_lowerright, sa,
+                                   dest_upperleft, da,
+                                   ConverterXYZ());
+        break;
+
     default:
         NEVER_REACHED("switch control expression \"BlendColorspace\" out of range");
     }
@@ -2203,6 +2470,8 @@ copyFromPyramidImageIf(typename PyramidI
                                              PyramidIntegerBits, PyramidFractionBits> ConverterLuv;
     typedef ConvertJCHPyramidToVectorFunctor<DestVectorType, PyramidVectorType,
                                              PyramidIntegerBits, PyramidFractionBits> ConverterJCH;
+    typedef ConvertXYZPyramidToVectorFunctor<DestVectorType, PyramidVectorType,
+                                             PyramidIntegerBits, PyramidFractionBits> ConverterXYZ;
 
     switch (BlendColorspace)
     {
@@ -2249,6 +2518,17 @@ copyFromPyramidImageIf(typename PyramidI
                                      ConverterJCH());
         break;
 
+    case XYZ:
+        if (Verbose >= VERBOSE_COLOR_CONVERSION_MESSAGES)
+        {
+            std::cerr << command << ": info: XYZ color conversion" << std::endl;
+        }
+        vigra::omp::transformImageIf(src_upperleft, src_lowerright, sa,
+                                     mask_upperleft, ma,
+                                     dest_upperleft, da,
+                                     ConverterXYZ());
+        break;
+
     default:
         NEVER_REACHED("switch control expression \"BlendColorspace\" out of range");
     }
