--- src/anneal.h.orig	2018-08-21 09:39:19 UTC
+++ src/anneal.h
@@ -441,6 +441,19 @@ class GDAConfiguration (protected)
                 bool hasHighWeightState = false;
                 for (unsigned int k = 0; k < localK; ++k) {
                     const double weight = (*stateProbabilities)[k];
+                    if (std::isnan(weight) || weight < 0. || weight > 1.) {
+#ifdef OPENMP
+#pragma omp critical(write_to_cerr)
+#endif
+                        std::cerr << command
+                          << ": warning: weight is out of range." 
+                          << std::endl 
+                          << "index: " << index 
+                          << "; k: " << k 
+                          << "; weight: " << weight 
+                          << std::endl;
+                        continue;
+                    }
                     totalWeight += weight;
                     if (weight > 0.99) {
                         hasHighWeightState = true;
@@ -449,8 +462,10 @@ class GDAConfiguration (protected)
                     estimateX += weight * static_cast<double>(state.x);
                     estimateY += weight * static_cast<double>(state.y);
                 }
-                estimateX /= totalWeight;
-                estimateY /= totalWeight;
+                if (totalWeight != 0.0) {
+                    estimateX /= totalWeight;
+                    estimateY /= totalWeight;
+                }
 
                 vigra::Point2D newEstimate(vigra::NumericTraits<int>::fromRealPromote(estimateX),
                                            vigra::NumericTraits<int>::fromRealPromote(estimateY));
