--- src/enblend.cc.orig	2017-10-29 15:53:13 UTC
+++ src/enblend.cc
@@ -119,6 +119,7 @@ std::optional<std::string> OutputMaskFileName;
 int Verbose = 0;                //< default-verbosity-level 0
 int ExactLevels = 0;            // 0 means: automatically calculate maximum
 bool OneAtATime = true;
+bool CopyMetadata = true;
 boundary_t WrapAround = OpenBoundaries;
 bool GimpAssociatedAlphaHack = false;
 blend_colorspace_t BlendColorspace = UndeterminedColorspace;
@@ -240,10 +241,11 @@ void dump_global_variables(const char* file, unsigned 
         "+ " << file << ":" << line << ": state of global variables\n" <<
         "+ Verbose = " << Verbose << ", option \"--verbose\"\n" <<
         "+ OutputFileName = <" << OutputFileName << ">\n" <<
-        "+ OutputMaskFileName = <" << OutputMaskFileName.value_or("<not defined>") << ">\n" <<
+        "+ OutputMaskFileName = <" << OutputMaskFileName.get_value_or("<not defined>") << ">\n" <<
         "+ ExactLevels = " << ExactLevels << "\n" <<
         "+ UseGPU = " << UseGPU << "\n" <<
         "+ OneAtATime = " << enblend::stringOfBool(OneAtATime) << ", option \"-a\"\n" <<
+        "+ CopyMetadata = " << enblend::stringOfBool(CopyMetadata) << ", option \"--[no-]metadata\"\n" <<
         "+ WrapAround = " << enblend::stringOfWraparound(WrapAround) << ", option \"--wrap\"\n" <<
         "+ GimpAssociatedAlphaHack = " << enblend::stringOfBool(GimpAssociatedAlphaHack) <<
         ", option \"-g\"\n" <<
@@ -491,7 +493,8 @@ void sigint_handler(int sig)
 
 
 enum AllPossibleOptions {
-    VersionOption, PreAssembleOption /* -a */, NoPreAssembleOption, HelpOption, LevelsOption,
+    VersionOption, PreAssembleOption /* -a */, NoPreAssembleOption,
+    MetadataOption, NoMetadataOption, HelpOption, LevelsOption,
     OutputOption, OutputMaskOption, VerboseOption, WrapAroundOption /* -w */,
     CheckpointOption /* -x */, CompressionOption, LZWCompressionOption,
     BlendColorspaceOption, FallbackProfileOption,
@@ -519,6 +522,12 @@ bool contains(const OptionSetType& optionSet,
 }
 
 
+static void lcmsErrorHandler(cmsContext context, cmsUInt32Number errorCode, char const *errorText)
+{
+    std::cerr << "lcms: error (" << errorCode << "): " << errorText << std::endl;
+}
+
+
 /** Warn if options given at the command line have no effect. */
 void warn_of_ineffective_options(const OptionSetType& optionSet)
 {
@@ -768,6 +777,8 @@ process_options(int argc, char** argv)
         PreferGpuId,
         PreAssembleId,
         NoPreAssembleId,
+        MetadataId,
+        NoMetadataId,
         CoarseMaskId,
         FineMaskId,
         OptimizeMaskId,
@@ -811,6 +822,8 @@ process_options(int argc, char** argv)
         {"preassemble", no_argument, 0, PreAssembleId}, // dash-less form: not documented, not deprecated
         {"no-pre-assemble", no_argument, 0, NoPreAssembleId},
         {"no-preassemble", no_argument, 0, NoPreAssembleId}, // dash-less form: not documented, not deprecated
+        {"metadata", no_argument, 0, MetadataId},
+        {"no-metadata", no_argument, 0, NoMetadataId},
         {"coarse-mask", optional_argument, 0, CoarseMaskId},
         {"fine-mask", no_argument, 0, FineMaskId},
         {"optimize", no_argument, 0, OptimizeMaskId},
@@ -1400,6 +1413,16 @@ process_options(int argc, char** argv)
             optionSet.insert(NoPreAssembleOption);
             break;
 
+        case MetadataId:
+            CopyMetadata = true;
+            optionSet.insert(MetadataOption);
+            break;
+
+        case NoMetadataId:
+            CopyMetadata = false;
+            optionSet.insert(NoMetadataOption);
+            break;
+
         case BlendColorspaceId:
             if (optarg != nullptr && *optarg != 0) {
                 std::string name(optarg);
@@ -1761,6 +1784,8 @@ int main(int argc, char** argv)
 
     //< layer-selector all-layers
     LayerSelection.set_selector(selector::find_by_id(selector::id_t::AllLayersId)->get());
+
+    cmsSetLogErrorHandler(lcmsErrorHandler);
 
     if (!getopt_long_works_ok())
     {
